/*******************************************************************************
 * Copyright (c) 2016, 2017 Oracle and/or its affiliates. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
 * which accompanies this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 * Roman Grigoriadi
 ******************************************************************************/

package org.eclipse.yasson.internal.serializer;

import org.eclipse.yasson.internal.JsonbContext;
import org.eclipse.yasson.internal.Marshaller;
import org.eclipse.yasson.internal.model.JsonBindingModel;
import org.eclipse.yasson.internal.model.JsonContext;

import javax.json.bind.serializer.JsonbSerializer;
import javax.json.bind.serializer.SerializationContext;
import javax.json.stream.JsonGenerator;

/**
 * Serializes an object with user defined serializer.
 *
 * @author Roman Grigoriadi
 * @param <T> type of serializer
 */
public class UserSerializerSerializer<T> implements JsonbSerializer<T> {

    private final JsonbSerializer<T> userSerializer;

    private JsonBindingModel model;

    /**
     * Create instance of current item with its builder.
     *
     * @param model model
     * @param userSerializer user serializer
     */
    public UserSerializerSerializer(JsonBindingModel model, JsonbSerializer<T> userSerializer) {
        this.model = model;
        this.userSerializer = userSerializer;
    }

    @Override
    public void serialize(T obj, JsonGenerator generator, SerializationContext ctx) {
        if (model.getContext() == JsonContext.JSON_OBJECT) {
            generator.writeKey(model.getWriteName());
        }
        JsonbContext jsonbContext = ((Marshaller) ctx).getJsonbContext();
        try {
            jsonbContext.addProcessedType(model.getType());
            userSerializer.serialize(obj, generator, ctx);
        } finally {
            jsonbContext.removeProcessedType(model.getType());
        }
    }
}
